Stan Math Library: stan/math/prim/scal/prob/bernoulli_logit_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

bernoulli_logit_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_BERNOULLI_LOGIT_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_BERNOULLI_LOGIT_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_bounded.hpp>
9 #include <stan/math/prim/scal/err/check_finite.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/inv_logit.hpp>
13 #include <stan/math/prim/scal/fun/log1m.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <boost/random/bernoulli_distribution.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <cmath>
19
20 namespace stan {
21
22 namespace math {
23
24 // Bernoulli(n|inv_logit(theta)) [0 <= n <= 1; -inf <= theta <= inf]
25 // FIXME: documentation
26 template <bool propto, typename T_n, typename T_prob>
27 typename return_type<T_prob>::type
28 bernoulli_logit_log(const T_n& n, const T_prob& theta) {
29 static const char* function("stan::math::bernoulli_logit_log");
30 typedef typename stan::partials_return_type<T_n, T_prob>::type
31 T_partials_return;
32
33 using stan::is_constant_struct;
34 using stan::math::check_not_nan;
35 using stan::math::check_bounded;
36 using stan::math::value_of;
37 using stan::math::check_consistent_sizes;
38 using stan::math::include_summand;
39 using stan::math::log1p;
40 using stan::math::inv_logit;
41 using std::exp;
42
43 // check if any vectors are zero length
44 if (!(stan::length(n)
45 && stan::length(theta)))
46 return 0.0;
47
48 // set up return value accumulator
49 T_partials_return logp(0.0);
50
51 // validate args (here done over var, which should be OK)
52 check_bounded(function, "n", n, 0, 1);
53 check_not_nan(function, "Logit transformed probability parameter", theta);
54 check_consistent_sizes(function,
55 "Random variable", n,
56 "Probability parameter", theta);
57
58 // check if no variables are involved and prop-to
59 if (!include_summand<propto, T_prob>::value)
60 return 0.0;
61
62 // set up template expressions wrapping scalars into vector views
63 VectorView<const T_n> n_vec(n);
64 VectorView<const T_prob> theta_vec(theta);
65 size_t N = max_size(n, theta);
66 OperandsAndPartials<T_prob> operands_and_partials(theta);
67
68 for (size_t n = 0; n < N; n++) {
69 // pull out values of arguments
70 const int n_int = value_of(n_vec[n]);
71 const T_partials_return theta_dbl = value_of(theta_vec[n]);
72
73 // reusable subexpression values
74 const int sign = 2*n_int-1;
75 const T_partials_return ntheta = sign * theta_dbl;
76 const T_partials_return exp_m_ntheta = exp(-ntheta);
77
78 // Handle extreme values gracefully using Taylor approximations.
79 static const double cutoff = 20.0;
80 if (ntheta > cutoff)
81 logp -= exp_m_ntheta;
82 else if (ntheta < -cutoff)
83 logp += ntheta;
84 else
85 logp -= log1p(exp_m_ntheta);
86
87 // gradients
88 if (!is_constant_struct<T_prob>::value) {
89 static const double cutoff = 20.0;
90 if (ntheta > cutoff)
91 operands_and_partials.d_x1[n] -= exp_m_ntheta;
92 else if (ntheta < -cutoff)
93 operands_and_partials.d_x1[n] += sign;
94 else
95 operands_and_partials.d_x1[n] += sign * exp_m_ntheta
96 / (exp_m_ntheta + 1);
97 }
98 }
99 return operands_and_partials.value(logp);
100 }
101
102 template <typename T_n,
103 typename T_prob>
104 inline
105 typename return_type<T_prob>::type
106 bernoulli_logit_log(const T_n& n,
107 const T_prob& theta) {
108 return bernoulli_logit_log<false>(n, theta);
109 }
110
111 } // namespace math
112 } // namespace stan
113 #endif
check_not_nan.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::sign
int sign(const T &z)
Definition: sign.hpp:9
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::math::inv_logit
fvar< T > inv_logit(const fvar< T > &x)
Definition: inv_logit.hpp:15
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::check_bounded
bool check_bounded(const char *function, const char *name, const T_y &y, const T_low &low, const T_high &high)
Return true if the value is between the low and high values, inclusively.
Definition: check_bounded.hpp:95
stan::math::bernoulli_logit_log
return_type< T_prob >::type bernoulli_logit_log(const T_n &n, const T_prob &theta)
Definition: bernoulli_logit_log.hpp:28
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
check_bounded.hpp
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::log1p
fvar< T > log1p(const fvar< T > &x)
Definition: log1p.hpp:16
value_of.hpp
inv_logit.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.